#include <syscall.h>

isize read(i32 fd, void* buf, usize size)
{
	isize ret;
	asm volatile (
		"syscall"
		: "=a" (ret)
		: "a" (0), "D" (fd), "S" (buf), "d" (size)
		: "rcx", "r11", "memory"
	);
	return ret;
}

isize write(i32 fd, const void* buf, usize size)
{
	isize ret;
	asm volatile (
		"syscall"
		: "=a" (ret)
		: "a" (1), "D" (fd), "S" (buf), "d" (size)
		: "rcx", "r11", "memory"
	);
	return ret;
}

void* mmap(void* addr, usize length, i32 prot, i32 flags, i32 fd, isize offset)
{
	register i32 r10 asm ("r10") = flags;
	register i32 r8 asm ("r8") = fd;
	register isize r9 asm ("r9") = offset;

	void* ret;
	asm volatile (
		"syscall"
		: "=a" (ret)
		: "a" (9), "D" (addr), "S" (length), "d" (prot), "r" (r10), "r" (r8), "r" (r9)
		: "rcx", "r11", "memory"
	);
	return ret;
}

i32 mprotect(void* addr, usize len, i32 prot)
{
	i32 ret;
	asm volatile (
		"syscall"
		: "=a" (ret)
		: "a" (10), "D" (addr), "S" (len), "d" (prot)
		: "rcx", "r11", "memory"
	);
	return ret;
}

i32 ftruncate(i32 fd, isize length)
{
	i32 ret;
	asm volatile (
		"syscall"
		: "=a" (ret)
		: "a" (77), "D" (fd), "S" (length)
		: "rcx", "r11", "memory"
	);
	return ret;
}

i32 memfd_create(const char* name, u32 flags)
{
	i32 ret;
	asm volatile (
		"syscall"
		: "=a" (ret)
		: "a" (319), "D" (name), "S" (flags)
		: "rcx", "r11", "memory"
	);
	return ret;
}
